//	DrawingPanel.h - The main drawing panel.
//	----------------------------------------------------------------------------
//	This file is part of Motion Composition Tool, a simple application for
//	creating simple abstract animations.
//	Copyright (c) 2007 Niall Moody.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//	----------------------------------------------------------------------------

#ifndef DRAWINGPANEL_H_
#define DRAWINGPANEL_H_

#include "juce.h"
#include <list>

//------------------------------------------------------------------------------
///	Represents a polygon at a single point.
struct Polygon
{
	///	X positions of the 4 points.
	float x[4];
	///	Y positions of the 4 points.
	float y[4];

	///	This point's time position.
	float time;

	///	So we can sort polys correctly.
	bool operator<(const Polygon& other)
	{
		return time < other.time;
	};
};

//------------------------------------------------------------------------------
///	The main drawing panel.
class DrawingPanel : public Component,
					 public Timer,
					 public ChangeBroadcaster
{
  public:
	///	Constructor.
	DrawingPanel();
	///	Destructor.
	~DrawingPanel();

	///	Draws the panel's current state.
	void paint(Graphics& g);

	///	So we can tell if the user's dragging a point.
	void mouseDown(const MouseEvent& event);
	///	So we can tell if the user's dragging a point.
	void mouseUp(const MouseEvent& event);
	///	For changing the shape of the various polygons
	void mouseDrag(const MouseEvent& event);

	///	So we can show/hide the draggable points.
	void keyPressed(const KeyPress& key);

	///	Used to animate the polygons.
	void timerCallback();

	///	Sets the length of time we're using.
	void setLength(float val);
	///	Sets the indexed polygon's colour.
	void setPolygonColour(int index, const Colour& col);
	///	Updates our current position in time.
	/*!
		This will update currentPolys by interpolating between the relevant
		frames (or taking the last one, if we're past it), and repaint the
		Component.
	 */
	void setTime(float val);

	///	Returns the colour of the indexed polygon.
	const Colour& getPolygonColour(int index) const {return colours[index];};
	///	Returns the current time.
	float getTime() const {return currentPosition;};

	///	Stores the indexed currentPoly in polygons.
	void storePoly(int index);

	///	Starts animation of the polygons.
	void startAnimation();
	///	Stops animation of the polygons.
	void stopAnimation();

	///	Clears the polygons list.
	void clear();
	///	Sets the indexed polygon list.
	void setPolygonList(int index, const std::list<Polygon>& polysList);
	///	Returns the indexed polygon list.
	const std::list<Polygon>& getPolygonList(int index) const;

	///	Returns the length.
	float getLength() const {return length;};

	///	Method to sort Polygons.
	static bool comparePolys(Polygon first, Polygon second);
	///	Method to find Polygons with matching times.
	static bool compareTimes(Polygon poly);
  private:
	///	Helper method to update currentPolys by interpolating between values in polygons.
	/*!
		\param Which polygon needs updating.
	 */
	void updateCurrentPolys(int index);

	///	The length of the time line.
	float length;
	///	Current position in the time line.
	float currentPosition;

	///	The current polygons' positions (as drawn on screen).
	Polygon currentPolys[4];

	///	Lists of all the 4 polygons' positions.
	std::list<Polygon> polygons[4];

  	///	The 4 polygons' colours.
	Colour colours[4];

	///	Which polygon (if any) is currently being dragged.
	int draggedPoly;
	///	Which point on the selected point is currently being dragged.
	int draggedPoint;

	///	Whether to draw the points or not.
	bool showPoints;

	///	Used to get the correct time for animation.
	int64 ticksPerSecond;
	///	Used to get the correct time for animation.
	int64 startTicks;
};

#endif
